/* The complete elliptic integral of the first kind */
#include <math.h>
#include <complex.h>
#include <float.h>

double complex _agm(double complex a, double complex b);
double complex _cEllipticK(double complex);

/** The arithmetic-geometric mean
 _agm(a, b) returns the arithmetic-geometric mean agm(a, b) of two numbers a and b.
 a_{n+1} &=& \frac{1}{2}(a_n + b_n)\\
 b_{n+1} &=&  \sqrt{a_n b_n}
 */
double complex
_agm(double complex a, double complex b) {
    double e;
    double complex m;
    if ((cabs(a) == 0) || (cabs(b) == 0) || (a == -b)) return 0;
    e = 2.0 * DBL_EPSILON * fmax(cabs(a), cabs(b));
    do {
        m = csqrt(a * b);
        a = (a + b) * 0.5;
        b = cabs(a + m) == cabs(a - m) ? cimag(m / a) > 0 ? -m : m : cabs(a + m) < cabs(a - m) ? -m : m;
    } while (cabs(a - b) > e);
    return (a + b) * 0.5;
}

/** The complete elliptic integral of the first kind 
 _cEllipticK(m) returns the complete elliptic integral of the first kind K(m)
 The argument m is the parameter m = k^2, where k is the elliptic modulus.
 K(m) = \int_0^{\pi / 2} \frac{1}{ \sqrt{1 - m \sin^2  \theta }} d \theta */
double complex
_cEllipticK(double complex m) {
    double complex c;
    c = csqrt(1.0 - m);
    if (cabs(c) == 0.0) return INFINITY;
    return M_PI_2 / _agm(1.0, c);
}

#ifdef DEBUG
#include <stdio.h>
#include <stdlib.h>
char * help = "Name\n\
    cEllipticK -- returns the complete elliptic integral of the first kind.\n\
    Rev. 1.2 (Jan. 7, 2023) (c) 2022, Takayuki HOSODA (aka Lyuka)\n\
Usage\n\
    cEllipticK m.re\n\
    cEllipticK m.re m.im\n\
    cEllipticK range_re range_im samples\n";
int
main(int argc, char *argv[]) {
    double complex m;
    double complex c;
    double         x, y; 
    int            n = 801;
    int            r, j;
    if (argc == 2) {
        m = (atof(argv[1]));
        c = _cEllipticK(m);
        printf("_cEllipticK(%+g %+gi) = %+.16g %+.16gi\n", creal(m), cimag(m), creal(c), cimag(c));
    }else if (argc == 3) {
        m = (atof(argv[1]) + I * atof(argv[2]));
        c = _cEllipticK(m);
        printf("_cEllipticK(%+g %+gi) = %+.16g %+.16gi\n", creal(m), cimag(m), creal(c), cimag(c));
    } else if (argc == 4) {
        x = atof(argv[1]);
        y = atof(argv[2]);
        n = atoi(argv[3]);
        if (n < 10) n = 10;
        printf("#Computed by \"_cEllipcK Rev.1.2 (Jan. 7, 2023) (c) 2022, Takayuki HOSODA (aka Lyuka)\n");
        for (r = 0 ; r <= n; r++) {
            for (j = 0 ; j <= n; j++) {
                m = (-x * r + (n - r) * x) / n + I * ((-y * j + (n -j) * y) / n);
                c = _cEllipticK(m);
                printf("%.16g\t%.16g\t%.16g\t%.16g\n",creal(m), cimag(m), cabs(c), carg(c));
            }
            printf("\n");
        }
    } else {
        printf("%s\n", help);
    }
    return(0);
}
#endif
